/*
    server.c - OSS trampoline sample server

    Copyright (C) 2000 Ram Sivrasubramanium <ram@linux-workshop.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "dsproxy.h"


/*
 * read exactly "count" number of bytes from a file descriptor
 */
static int read_exact(
	int fd, 
	void *buf2, 
	size_t count)
{
	char *buf = buf2;
	int savedcount = count;
	size_t siz;
	
	while (count > 0) {
		siz = read(fd, buf, count);
		if (siz < 0) return(siz);
		
		buf += siz;
		count -= siz;
	}
	
	return(savedcount);
}


/*
 * establish network connection
 */ 
static int ListenOnTCPPort(
	int port)
{ 
	struct sockaddr_in addr;
	int sock;
	int one = 1;
	
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	
	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) return(-1);
	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) < 0) return(-1);
	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) return(-1);
	if (listen(sock, 5) < 0) return(-1);
	
	return(sock);
}


static int AcceptConnectionOnTCPPort(
	int port)
{
	int listensock, sock;
#if !defined( __socklen_t_defined )
	struct sockaddr_in addr;
	int addrlen = sizeof(addr);
#else
	struct sockaddr addr;
	socklen_t addrlen = sizeof(addr);
#endif 
	
	if ((listensock = ListenOnTCPPort(port)) < 0) return(-1);
	if((sock = accept(listensock, &addr, &addrlen)) < 0) return(-1);
	
	return(sock);
}


/*
 * simply copy bytes from dsproxy's /dev/dsp and reroute them across the network
 */
int main(
	int argc, 
	char **argv)
{
	int sock, fdi;
	char op;
	unsigned short len;
	int cmd, val;
	char buffer[DSPROXY_FIFO_SIZE];
	
	if ((sock = AcceptConnectionOnTCPPort(DSPROXY_TCP_PORT)) < 0) {
		fprintf(stderr, "Socket error, exiting.\n");
		exit(255);
	}
	
	if ((fdi = open("/dev/dsp", O_RDONLY)) < 0) {
		fprintf(stderr,"Error in opening /dev/dsp!\n");
		exit(255);
	}
	
	fprintf(stderr, "Client connected to server.\n");
	
	for( ; ; ) {
		read_exact(fdi, &op, sizeof(char));
		write(sock, &op, sizeof(char));
		switch (op) {
			case DSPROXY_STREAM_DSPDATA:
				read_exact(fdi, &len, sizeof(short));
				read_exact(fdi, buffer, (len == 0) ? 65536 : len);
				
				write(sock, &len, sizeof(short));
				write(sock, buffer, (len == 0) ? 65536 : len);  /* this is ok since /dev/dsp tags with exactly how many bytes follow */
				break;
			
			case DSPROXY_STREAM_IOCTL:
			case DSPROXY_MIXER_IOCTL:
				read_exact(fdi, &cmd, sizeof(int));
				read_exact(fdi, &val, sizeof(int));
				
				write(sock, &cmd, sizeof(int));
				write(sock, &val, sizeof(int));
				break;
			
			default:
				break;
		}	
	}
	
	exit(0);
}

